/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.15 $";
#endif

/*
 * Get system model name
 */

#include "defs.h"

/*
 * Read MODELFILE to get name of system model.
 */
extern Opaque_t GetModelFile()
{
    static char 		Buf[MAXPATHLEN];
    FILE 		       *fd;
    int 			Len;

    /*
     * Use model file if it exists.
     */
    if ((fd = fopen(MODELFILE, "r")) != NULL) {
	fgets(Buf, sizeof(Buf), fd);
	Len = strlen(Buf);
	if (Buf[Len-1] == '\n') 
	    Buf[Len-1] = C_NULL;
	return(Buf);
    }

    return(NULL);
}

/*
 * Run a command to get system model
 */
extern Opaque_t GetModelCmd()
{
#if	defined(MODEL_COMMAND)
    extern char 	       *ModelCommand[];

    return(RunCmds(ModelCommand, FALSE));
#else
    return((char *) NULL);
#endif	/* MODEL_COMMAND */
}

/*
 * Return the predefined symbol MODEL_NAME
 */
extern Opaque_t GetModelDef()
{
#if	defined(MODEL_NAME)
    return(MODEL_NAME);
#else	/* !MODEL_NAME */
    return((char *) NULL);
#endif	/* MODEL_NAME */
}

/*
 * Get system model by cobbling together something based on CPU info
 */
extern Opaque_t GetModelCpuInfo()
{
    CpuInfo_t		       *CpuInfo;
    static char			Buff[256];
    char		       *cp = Buff;
    char		       *Str;
    MCSIquery_t			Query;

    if (Buff[0])
	return Buff;

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_CPUINFO;
    if (mcSysInfo(&Query) != 0)
	return (char *) NULL;

    CpuInfo = (CpuInfo_t *) Query.Out;

    if (CpuInfo->Man) {
	strcpy(cp, CpuInfo->Man);
	cp += strlen(cp);
    }

    if (CpuInfo->Speed) {
	if (Buff[0]) strcat(cp++, " ");
	Str = GetClockStr(CpuInfo->Speed, (Large_t)MHERTZ);
	(void) snprintf(cp, &Buff[sizeof(Buff)]-cp, "%s", Str);
	cp += strlen(Str);
    }

    if (CpuInfo->Model) {
	if (Buff[0]) strcat(cp++, " ");
	(void) snprintf(cp, &Buff[sizeof(Buff)]-cp, "%s", CpuInfo->Model);
	cp += strlen(CpuInfo->Model);
    }

    SImsg(SIM_DBG, "GetModelCpuInfo: Model=<%s>", Buff);

    return (Buff[0]) ? Buff : (char *) NULL;
}

/*
 * Get system model
 */
extern int GetModel(Query)
     MCSIquery_t	      *Query;
{
    extern PSI_t	        GetModelPSI[];
    char		       *Str;

    if (Query->Op == MCSIOP_CREATE) {
	if (Str = PSIquery(PsiSwitch.GetModel)) {
	    Query->Out = (Opaque_t) strdup(Str);
	    Query->OutSize = strlen(Str);
	    return 0;
	}
    } else if (Query->Op == MCSIOP_DESTROY) {
	if (Query->Out && Query->OutSize)
	    (void) free(Query->Out);
	return 0;
    }

    return -1;
}

