/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.12 $
 */

/*
 * This file contains VESA EDID definetions common to SysInfo.
 *
 * EDID info from:
 *	Extended Display Identification Data (EDID) Standard
 *		Version 3, Revision Date: 11/13/97
 */

#ifndef __edid_h__
#define __edid_h__ 

/*
 * This macro is needed for all Byte2_t or larger variables to insure
 * the compiler does not increase allocated variable space for alignment
 * reasons and thus invalidate the variable location.
 */
#if	defined(HAVE_ATTRIBUTE_PACKED)
# define _PACK_			__attribute__ ((packed))
#else
# define _PACK_
#endif

/*
 * Basic types
 */
typedef unsigned char		Byte_t;		/* 1 byte/8 bit  */
typedef unsigned short		Byte2_t;	/* 2 byte/16 bit */
typedef unsigned int		Byte4_t;	/* 4 byte/32 bit */

/*
 * Standard EDID header should contain EDID_HEADER
 */
#define EDID_HEADER	{ 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00 }

/*
 * Monitor Data Tags identifying type of Descriptor Block (DataType)
 */
#define MDD_SERIAL	0xFF		/* Monitor Serial Number (ASCII) */
#define MDD_STRING	0xFE		/* General ASCII String */
#define MDD_RANGE	0xFD		/* Monitor Range Limits */
#define MDD_NAME	0xFC		/* Monitor Name (ASCII) */
#define MDD_COLORPT	0xFB		/* Color Point Data */
#define MDD_STDTIMING	0xFA		/* Add. Std Timing Identifications */
/* DataTypes of range 0x00 - 0x0F are reserved for Manufacturer use */

/*
 * Detailed Timing Descriptor
 */
typedef struct {
    Byte2_t	PixelClock;		/* Hz */
    Byte_t	HorActive1;		/* # Pixels */
    Byte_t	HorBlanking1;		/* # Pixels */
#if	defined(_BIT_FIELDS_LTOH)
    Byte_t	HorBlanking2 	: 4,
    		HorActive2 	: 4
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte_t	HorActive2 	: 4,
		HorBlanking2 	: 4
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
    Byte_t	VerActive2;		/* # Lines */
    Byte_t	VerBlanking2;		/* # Lines */
#if	defined(_BIT_FIELDS_LTOH)
    Byte_t	VerActive3 	: 4,
		VerBlanking3 	: 4
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte_t	VerBlanking3 	: 4,
    		VerActive3 	: 4
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
    Byte_t	HorSyncOffSet1;		/* # Pixels */
    Byte_t	HorSyncPulseWidth;	/* # Pixels */
#if	defined(_BIT_FIELDS_LTOH)
    Byte_t	VertSyncPulseWidth1 : 4,/* # lines */
    		VertSyncOffset1     : 4 /* # lines */
	_PACK_;
    Byte_t	VerSyncPulseWidth2 	: 2,
		VerSyncOffSet2 		: 2,
		HorSyncPulseWidth2 	: 2,
    		HorSyncOffSet2 		: 2
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte_t	VertSyncOffset1     : 4,/* # lines */
		VertSyncPulseWidth1 : 4 /* # lines */
	_PACK_;
    Byte_t	HorSyncOffSet2 		: 2,
		HorSyncPulseWidth2 	: 2,
		VerSyncOffSet2 		: 2,
		VerSyncPulseWidth2 	: 2
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
    Byte_t	HorImageSize1;		/* Horizontal Image Size (mm) */
    Byte_t	VerImageSize1;		/* Vertical Image Size (mm) */
#if	defined(_BIT_FIELDS_LTOH)
    Byte_t	VerImageSize2 : 4,
 		HorImageSize2 : 4
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte_t	HorImageSize2 : 4,
 		VerImageSize2 : 4
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
    Byte_t	HorBorder;		/* # Pixels */
    Byte_t	VerBorder;		/* # Lines */
#if	defined(_BIT_FIELDS_LTOH)
    Byte_t	Flags 		: 3,
		Composite	: 2,	/* 0=Analog 1=Bipolar Analog */
					/* 2=Digital Composite 3=Digital Sep */
		Type		: 2,	/* 0=Normal (not stereo) */
 		Interlaced 	: 1	/* 0=Non-Interlaced 1=Interlaced */
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte_t	Interlaced 	: 1,	/* 0=Non-Interlaced 1=Interlaced */
		Type		: 2,	/* 0=Normal (not stereo) */
		Composite	: 2,	/* 0=Analog 1=Bipolar Analog */
					/* 2=Digital Composite 3=Digital Sep */
		Flags 		: 3
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
} DetailTiming_t;

/*
 * Monitor Range Limits
 * Used when MonDescriptor_t.DataType == MDD_RANGE
 */
typedef struct {
    Byte_t	MinVertical;		/* Min. Vertical Rate (Hz) */
    Byte_t	MaxVertical;		/* Max. Vertical Rate (Hz) */
    Byte_t	MinHorizontal;		/* Min. Horizontal rate (KHz) */
    Byte_t	MaxHorizontal;		/* Max. Horizontal rate (KHz) */
    Byte_t	MaxPixelClock;		/* Max. Support. Pixel Clock (MHz/10)*/
    Byte_t	GTFreserved[8];		/* Reserved for GTF standard */
} MonRange_t;

/*
 * Monitor Descriptor Block
 * This is a general buffer that is interpretted based on DataType.
 */
typedef struct {
    Byte2_t	Flag1;			/* Should be == 0x0000 if descriptor */
    Byte_t	Reserved;		/* Reserved */
    Byte_t	DataType;		/* Type of data (see MDD_*) */
    Byte_t	Flag2;			/* Should be == 0x00 if descriptor */
    Byte_t	Data[13];		/* The data itself */
} Detail_t;

/*
 * Standard Timings consistant of 2-byte values
 */
typedef struct {
    Byte_t	HorActivePixels;	/* Horizontal Active Pixels (*8 pix) */
#if	defined(_BIT_FIELDS_LTOH)
    Byte_t	RefreshRate 	: 6,	/* Refesh Rate */
    		Aspect 		: 2	/* Aspect Ratio */
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte_t	Aspect 		: 2,	/* Aspect Ratio */
		RefreshRate 	: 6	/* Refesh Rate */
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
} StdTiming_t;

/*
 * EDID Version 1
 */
typedef struct {
    /* EDID Header */
    Byte_t	Header[8];		/* EDID Header */
    /*
     * Vendor / Product Identification 
     */
 					/* EISA Mfg ID code (2-bytes) */
#if	defined(_BIT_FIELDS_LTOH)
    Byte2_t	ManIDc3 	: 5,	/* Character 3 */
		ManIDc2 	: 5,	/* Character 2 */
		ManIDc1 	: 5,	/* Character 1 */
    		ManIDpad 	: 1	/* Padding */
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte2_t	ManIDpad 	: 1,	/* Padding */
		ManIDc1 	: 5,	/* Character 1 */
		ManIDc2 	: 5,	/* Character 2 */
		ManIDc3 	: 5	/* Character 3 */
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
    Byte2_t	ProductIdCode;		/* Product ID code */
    Byte4_t	Serial;			/* Serial number */
    Byte_t	ManWeek;		/* Week # manufactured */
    Byte_t	ManYear;		/* Year manufactured */
    /*
     * EDID Version / Revision 
     */
    Byte_t	EDIDversion;		/* EDID Version # */
    Byte_t	EDIDrevision;		/* EDID Revision # */
    /* 
     * Basic Display Parameters / Features 
     */
#if	defined(_BIT_FIELDS_LTOH)
    Byte_t	SigSepSync	: 1,	/* Seperate sync */
		SigCompSync 	: 1,	/* Composite Sync */
		SigSyncGreen 	: 1,	/* Sync on Green */
		SigUnused	: 2,	/* Unused */
		SigVoltLevel	: 2,	/* Voltage Level */
		SigType : 1		/* 0=Analog 1=Digital */
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte_t	SigType 	: 1,	/* 0=Analog 1=Digital */
		SigVoltLevel	: 2,	/* Voltage Level */
		SigUnused	: 2,	/* Unused */
		SigSyncGreen 	: 1,	/* Sync on Green */
		SigCompSync 	: 1,	/* Composite Sync */
    		SigSepSync	: 1	/* Seperate sync */
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
    Byte_t	HorImageSize;		/* Max Horizontal Image Size (cm) */
    Byte_t	VerImageSize;		/* Max Vertical Image Size (cm) */
    Byte_t	Gamma;			/* Display Transfer Characteristic */
#if	defined(_BIT_FIELDS_LTOH)
    Byte_t	HasGtf 		: 1, 	/* Supports GTF standard timings */
		HasPrefTiming 	: 1,	/* Preferred Timing in 1st Detail 
					   Timing Block */
		StdColorSpace 	: 1,	/* Uses std default color space */
		DisplayType 	: 2,	/* mono, color, etc */
		DpmsActiveOff 	: 1, 	/* Supports DPMS Active-Off */
		DpmsSuspend 	: 1,	/* Supports DPMS Suspend mode */
		DpmsStandBy 	: 1	/* Supports DPMS Stand-By mode */
	_PACK_;
#elif	defined(_BIT_FIELDS_HTOL)
    Byte_t	DpmsStandBy 	: 1,	/* Supports DPMS Stand-By mode */
		DpmsSuspend 	: 1,	/* Supports DPMS Suspend mode */
		DpmsActiveOff 	: 1, 	/* Supports DPMS Active-Off */
		DisplayType 	: 2,	/* mono, color, etc */
		StdColorSpace 	: 1,	/* Uses std default color space */
		HasPrefTiming 	: 1,	/* Preferred Timing in 1st Detail 
					   Timing Block */
		HasGtf 		: 1 	/* Supports GTF standard timings */
	_PACK_;
#endif	/* _BIT_FIELDS_LTOH */
    Byte_t	ColorChars[10];		/* Color Characteristics */
    /*
     *  Established Timings (3 Bytes)
     */
    Byte_t 	EstTimings[3];
    /*
     * Standard Timings (16 bytes)
     */
    StdTiming_t	StdTimings[8];
    /* 
     * Detailed Timing Descriptor Blocks (4 * 18 = 72 bytes)
     */
    Detail_t	Details[4];
    /* Misc */
    Byte_t	ExtFlag;		/* Extension Flag: # of following
					   128-byte EDID extension blocks */
    Byte_t	CheckSum;		/* 1-byte sum of all 128-bytes in
					   this EDID should == 0 */
} EDIDv1_t;

#define EDID_V1_SIZE		128	/* This better be 128-bytes */

/*
 * EDID Version 2
 */
typedef struct {
    /* EDID Info */
    Byte_t		EDIDversion : 4,	/* EDID Version */
			EDIDrevision : 4;	/* EDID Revision */
    /* Manufacturer / Product Info */
    				/* EISA Manufacturer ID code (2-bytes) */
    Byte2_t		ManIDpad : 1,		/* Padding */
			ManIDc1 : 5,		/* Character 1 */
			ManIDc2 : 5,		/* Character 2 */
			ManIDc3 : 5;		/* Character 3 */
    Byte_t		ProductIdCode[2]; /* Product ID code */
    Byte_t		ManWeek;	/* Week # manufactured */
    Byte2_t		ManYear;	/* Year manufactured */
    Byte_t		ManProduct[32];	/* Manufacturer + Product names */
    Byte_t		Serial[16];	/* Serial number */
    /* Basic Display Parameters/Features */
    Byte_t		PriPhysInt : 4,	/* Primary physical interface */
			SecPhysInt : 4;	/* Secondary physical interface */
    Byte_t		PriVidInt : 4,	/* Primary Video interface */
			SecVidInt : 4;	/* Secondary Video interface */
    	/* Analog Interface */
    Byte_t		AnalogInfo;	/* Analog interface info */
    Byte_t	       	PixelClock : 1,	/* Has Pixel Clock */
			Reserved0 : 7;
    Byte_t		Reserved1[2];
    	/* Digital Interface */
    Byte_t		DisplayEnableParity : 1, /* Display enabled */
			EdgeShiftClock : 1,	/* falling edge shift clock */
			RecvrUnits : 2,	/* # of data receiver units */
			ChSpeed : 4;	/* Channel Speed Exponent */
    Byte_t		MinChSpeed;	/* Min. channel speed */
    Byte_t		MaxChSpeed;	/* Max. channel speed */
    Byte_t		DigitalFormat;	/* Digital interface data format */
    Byte_t		ColorEncoding : 4,	/* Color encoding */
			SecColorEncoding : 4;	/* Secondary color encoding */
    Byte_t		DepthSub0 : 4,	/* Bit-Depth sub-channel 0 */
			DepthSub1 : 4;	/* Bit-Depth sub-channel 1 */
    Byte_t		DepthSub2 : 4,	/* Bit-Depth sub-channel 2 */
			DepthSub3 : 4;	/* Bit-Depth sub-channel 3 */
    	/* Not sure why this is repeated in the std */
    Byte_t		OthDepthSub0 : 4,/* Bit-Depth sub-channel 0 */
			OthDepthSub1 : 4;/* Bit-Depth sub-channel 1 */
    Byte_t		OthDepthSub2 : 4,/* Bit-Depth sub-channel 2 */
			OthDepthSub3 : 4;/* Bit-Depth sub-channel 3 */
 	/* Display Device Description */
    Byte_t		DispTechType : 4,	/* Display Technology Type */
			DispTypeSubType : 4;	/*    " subtype */
    Byte_t		DisplayType : 1,	/* Mono, Color */
			SelChromat : 1,	/* Selectable Display chromaticity */
			CondUpdate : 1,	/* Conditional Update */
			ScanOrient : 2,	/* Scan Orientation */
			DispBg : 1,	/* Display Background */
			PhysImp : 2;	/* Physical Implementation */
    Byte_t		DpmsStandBy : 1,/* Supports DPMS Stand-By mode */
			DpmsSuspend : 1,/* Supports DPMS Suspend mode */
			DpmsActiveOff : 1, /* Supports DPMS Active-Off */
			DpmsOff : 1,	/* Supports DPMS Off */
			Stereo : 3,	/* Stereo Support info */
			Reserved2 : 1;
    Byte_t		AudioInType : 1, 	/* Audio Input Type */
			AudioInInt : 2,		/* Audio Input Interface */
			AudioOutType : 1,	/* Audio Output Type */
			AudioOutInt : 2,	/* Audio Output Interface */
			VideoInType : 2;	/* Video Input Type */
    Byte_t		TouchScreen : 1,	/* Contains touch screen */
			LightPen : 1,		/* Supports light pen */
			LumProbe : 1,		/* Supports Luminance Probe */
			Colorimeter : 1,	/* Supports colorimeter */
			AdjOrient : 1,		/* Adjustable orientation */
			Reserved3 : 3;
    	/* Display Response Time */
    Byte_t		RiseExp : 4,	/* Rise Time Exponenet */
			RiseSecs : 4;	/* Rise Time in seconds */
    Byte_t		FallExp : 4,	/* Fall Time Exponent */
			FallSecs : 4;	/* Fall Time in seconds */
 	/* Color/Luminance */
    Byte_t		GammaWhite;	/* White Gamma */
    Byte_t		GammaColor0;	/* Color 0 (red) Gamma */
    Byte_t		GammaColor1;	/* Color 1 (green) Gamma */
    Byte_t		GammaColor2;	/* Color 2 (blue) Gamma */
    Byte2_t		MaxLum;		/* Max Luminance */
    Byte_t		StdRGB : 1,	/* Standard RGB model */
			AdjGamma : 1,	/* Adjustable Gamma */
			Reserved4 : 6;
    Byte_t		LumOffSet;	/* Luminance Offset */
    	/* Chromaticity */
    Byte_t		Chromaticity[10];
    Byte_t		WhitePoints[10];
    	/* Display Spatial Description */
    Byte2_t		HorImageSize;	/* Max Horizontal Image Size (mm) */
    Byte2_t		VerImageSize;	/* Max Vertical Image Size (mm) */
    Byte2_t		MaxHorAddr;	/* Max. Hor. Addressibility (pixels) */
    Byte2_t		MaxVerAddr;	/* Max. Ver. Addressibility (pixels) */
    Byte_t		HorPitch;	/* Max Hor. Pixel Pitch */
    Byte_t		VerPitch;	/* Max Ver. Pixel Pitch */
    	/* GTF */
    Byte_t		GTF;
    	/* Timing codes */
    Byte_t		ExtFlag : 1,	/* 1 more EDID block follows */
			HasPrefTiming : 1,	/* Preferred Timing in 1st 
						   Detail Timing Block */
			HasLumTab : 1,		/* Luminance Table provided */
			NumFreqRanges : 3,	/* # of Frequency Ranges */
			NumDetailRange : 2;	/* # of Detail Range Limits */
    Byte_t		NumTimingCodes : 5,	/* # of Timing Codes listed */
			NumDetailTimings : 3;	/* # of De3tailed Timings */
    	/* Tables */
    Byte_t		Tables[127];
    Byte_t		CheckSum;	/* 1-byte sum of all == 0 */
} EDIDv2_t;
#define EDID_V2_SIZE		256	/* Better be 256 */

#endif	/* __edid_h__ */
