/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.2 $";
#endif

/*
 * Printer AIX functions
 */

#include "defs.h"

/*
 * The following AIX commands produce printer queue info
 *	/usr/lib/lpd/pio/etc/piolsvp
 *	/usr/bin/lsallq
 * The details are in /etc/qconfig.
 */

static char *PiolsvpPath[] = {
    "/usr/lib/lpd/pio/etc/piolsvp", NULL
};

/*
 * Use piolsvp(1) to obtain printer info
 */
static int PrinterPiolsvp(PrinterTree, SearchExp)
     Printer_t		      **PrinterTree;
     char		      **SearchExp;
{
    Printer_t		       *Printer;
    char		       *Piolsvp;
    static char			Cmd[1024];
    FILE		       *Fp;
    static char			Buff[1024];
    char		       *cp;
    char		       *bp;
    char		       *Field0;
    char		       *Field1;
    char		       *Field2;

    Piolsvp = (char *) CmdFind(PiolsvpPath);
    if (!Piolsvp) {
	SImsg(SIM_DBG, "PrinterPiolsvp: Cannot find \"piolsvp\" command.");
	return -1;
    }
    snprintf(Cmd, sizeof(Cmd), "%s -v", Piolsvp);
    SImsg(SIM_DBG, "PrinterPiolsvp: Command=<%s>", Cmd);

    Fp = popen(Cmd, "r");
    if (!Fp) {
	SImsg(SIM_DBG, "%s: Command start failed: %s", Cmd, SYSERR);
	return -1;
    }

    /*
     * Input looks like:
     *
     *# PRINT QUEUE          PRINTER              DESCRIPTION
     *  mcp1                 @dune                MagniComp Printer          
     *  mcp1sysv             @dune                MagniComp Printer SysV    
     *  localascii           lp0                  generic (ASCII)             
     *  localpcl             lp0                  generic (PCL Emulation)     
     *  localps              lp0                  generic (PostScript)        
     */
    while (bp = fgets(Buff, sizeof(Buff), Fp)) {
	if (Buff[0] == '#')
	    continue;
	if (cp = strchr(bp, '\n'))
	    *cp = CNULL;

	SImsg(SIM_DBG, "PrinterPiolsvp: Input=<%s>", Buff);

	/* Set Field0 */
	for (cp = bp; cp && *cp && isspace(*cp); ++cp);	/* Skip space */
	Field0 = cp;
	for (cp = Field0; cp && *cp && !isspace(*cp); ++cp); /* Find end */
	if (!cp || !*cp)
	    continue;		/* Premature */
	else
	    *cp = CNULL;	/* Terminate Field0 */
	bp = cp + 1;

	/* Set Field1 */
	for (cp = bp; cp && *cp && isspace(*cp); ++cp);	/* Skip space */
	Field1 = cp;
	for (cp = Field1; cp && *cp && !isspace(*cp); ++cp); /* Find end */
	if (!cp || !*cp)
	    continue;		/* Premature */
	else
	    *cp = CNULL;	/* Terminate Field */
	bp = cp + 1;

	/* Set Field2 */
	for (cp = bp; cp && *cp && isspace(*cp); ++cp);	/* Skip space */
	Field2 = cp;
	/* Start from end and trim space */
	for (cp = &Field2[strlen(Field2)-1]; cp && *cp && isspace(*cp); --cp);
	if (cp && cp+1 && isspace(*(cp+1)))
	    *(cp+1) = CNULL;	/* Terminate Field */

	Printer = PrinterCreate(NULL);
	Printer->Queue = strdup(Field0);
	PrinterAdd(PrinterTree, Printer);

	if (Field1)
	    if (*Field1 == '@') {
		Printer->Server = strdup(Field1+1);
	    } else {
		Printer->Device = strdup(Field1);
		Printer->Proto = PP_LOCAL;
	    }
	if (Field2)
	    Printer->Desc = strdup(Field2);
	
	SImsg(SIM_DBG, "PrinterPiolsvp: Queue=<%s> Server=<%s> Desc=<%s>",
	      PRTS(Printer->Queue), PRTS(Printer->Server),
	      PRTS(Printer->Desc));
    }

    pclose(Fp);
}

/*
 * Build Printer info on AIX
 */
extern Opaque_t BuildPrinterAIX(PrinterTree, SearchExp)
     Printer_t		      **PrinterTree;
     char		      **SearchExp;
{
    int				Found = 0;

    if (!PrinterTree)
	return (Opaque_t) -1;

    Found = PrinterPiolsvp(PrinterTree, SearchExp);

    return (Opaque_t) Found;
}
