#
# $Id: uiTableAdd.pm,v 1.8 2002/03/16 01:33:57 mcooper Exp $
#
# UI Table Add functions
#

package uiTableAdd;

require 5.003;

require Exporter;

use mcDebug;
use mcGetSize;
use uiMisc;
use uiGlobals;
use uiPath;
use uiMeter;
use uiFormat;
use uiArrange;

#
# Definetions used when displaying Key definetions
#
my $KeyDefs = [
		   {Key=>'AltName', 	Value=>'AKA Name'},
		   {Key=>'Name', 	NoShow=>1},
	      ];

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   TableAdd TableAddProps
		  );
}

#
# Get the key definetion from the table
#
sub GetKeyDef {
    my $Key = shift;
    my $Class = shift;

    my @Defs = @$KeyDefs;
    for (my $i = 0; $i <= $#Defs; ++$i) {
	if ($Defs[$i]->{Class} && !Eq($Class, $Defs[$i]->{Class})) {
	    next;
	}
	if (Eq($Key, $Defs[$i]->{Key})) {
	    return $Defs[$i];
	}
    }
}

#
# Add properties entry to the Tk Tree widget
#
sub TreeAddPropsEntry {
    my $Tree = shift;
    my $Path = shift;
    my $Str = shift;
    my $Hide = shift;

    if (!$Tree || !Tk::Exists($Tree)) {
	return;
    }

    if ($Tree->infoExists($Path)) {
	Debug("TreeAddPropsEntry: Path exists <%s>", $Path);
	return;
    }

    $Tree->add($Path, -text=>$Str);
    $Tree->setmode($Path, 'none');
    if ($Hide > 0) {
	$Tree->hide('entry', $Path);
    } else {
	$Tree->show('entry', $Path);
    }
}

#
# Add properties of Entry to $Table
#
sub TableAddProps {
    my $Args = shift;
    my $Widget = $Args->{Widget};
    my $Entry = $Args->{Entry};
    my $ParentPath = $Args->{Path};

    if ($Entry !~ /HASH/) {
	return;
    }

    # There's no useful property info for these
    if ($ParentPath =~ /SysConf/ || $ParentPath =~ /Kernel/) {
	return;
    }

    my $Table = uiPath::PathTable($ParentPath);

    my $Hide = 0;
    if ($Entry =~ /DevInfo/ || $Entry =~ /SoftInfo/ || $Entry =~ /PartInfo/ ||
	$Entry =~ /PrinterInfo/ || $Entry =~ /CacheInfo/) {
	$Hide = 1;
    }

    my @p = split($PathSep, $ParentPath);
    my $Class = $p[0];

    my $OffSet = 25;
    my $NewTable;

    foreach my $Name (keys %$Entry) {
	my $Value = $Entry->{$Name};

##	Debug("TableAddProps: Name=<%s> Value=<%s> ref=<%s>",
##	      $Name, $Value, ref($Value));

	if (IsHash($Value)) {
	    #
	    # Decode each value of $Value as a hash
	    #
	    foreach my $key (keys %$Value) {
		if (!length($Value->{$key})) {
		    next;
		}
		my $Tag;
		# Use the object's Desc function to lookup its description
		if (!$Tag && exists($Value->{'Desc'})) {
		    $Tag = $Value->Desc($key);
		}
		# Change display tag 'DescXXXX' to 'Description'
		if (!$Tag && $key =~ /Desc.[0-9]/) {
		    $Tag = 'Description';
		}
		# Default to the key
		if (!$Tag) {
		    $Tag = $key;
		}

		#
		# Create a unique key based on $Tag and it's value
		# $Tag is used to insure alpha sorted order is used
		# later when $Tag is displayed to user.
		#
		my $pkey = $Tag . $Value->{$key};
		$pkey =~ s/$PathSep//g;
		# Path to this entry
		my $Path = $ParentPath . $PathSep . $pkey;

		my $Sum = sprintf("%-*s %s",
				  $OffSet, ucfirst($Tag) . ':',
				  $Value->{$key});

##		Debug("TableAddProps:     Desc=<%s> Value=<%s> HASH",
##		      $Tag, $Value->{$key});

		$NewTable->{$Path} = $Table->{$Path} = {
							Summary=>$Sum,
							Hide=>$Hide,
							Desc=>$Tag,
							Value=>$Value->{$key}
						       };
	    }
	} else {
	    #
	    # Decode scalar and array types
	    #
	    my $KeyDef = $Name;
	    if (exists($Entry->{'Desc'})) {
		$KeyDef = $Entry->Desc($Name);
		if (!length($KeyDef)) {
		    next;
		}
	    }
	    if (!$KeyDef) {
		if (my $Def = GetKeyDef($Name, $Class)) {
		    if ($Def->{NoShow}) {
			next;
		    }
		    if ($Def->{Value}) {
			$KeyDef = $Def->{Value};
		    }
		}
	    }
	    if (!$KeyDef) {
		$KeyDef = $Name;
	    }

	    #
	    # Create the $Path value using $KeyDef to insure alpha sorted
	    # order when displayed to user.
	    #
	    my $Path = $ParentPath . $PathSep . $KeyDef;
	    if (Eq($KeyDef, 'Disk Usage')) {
		$Value = GetSizeStr($Value);
	    } elsif (lc($KeyDef) =~ /date/ && ($Entry->{$Name} !~ /.[\-\/]/)) {
		$Value = GetDate($Entry->{$Name});
	    } else {
		$Value = GetValue($Entry->{$Name});
	    }
	    if (!length($Value)) {
		next;
	    }

	    my $Sum = sprintf("%-*s %s", $OffSet, $KeyDef . ':', $Value);
	    #
	    # $Table is the global table which tracks current valid entries.
	    # $NewTable contains just this runs new additions.
	    #
	    $NewTable->{$Path} = $Table->{$Path} = {
						    Summary=>$Sum,
						    Hide=>$Hide,
						    Desc=>$KeyDef,
						    Value=>$Value
						   };
	}
    }

    if (FormatIs($FT_TREE) && $Widget && Tk::Exists($Widget)) {
	#
	# Add the entries in sorted order
	#
	foreach my $Path (sort keys %$NewTable) {
	    TreeAddPropsEntry($Widget, $Path,
			      $NewTable->{$Path}->{'Summary'},
			      $NewTable->{$Path}->{'Hide'});
	}
    }
}

#
# Build a Table by adding $Class->{Entires} or $Obj to it.
#
sub TableAdd {
    my $Args = shift;
    my $Class = $Args->{Class};
    my $Obj = $Args->{Obj};
    my $Table;

    if (!$Class) {
	Debug("TableAdd: No class specified.");
	return;
    }

    if (!$Obj) {
	$Obj = $Class->{Entries};
    }

    #
    # Insure the Class is itself present.
    #
    $Table->{$Class->{Name}}->{Obj} = $Class->{Name};

    #
    # If this is Hardware and Arrange is by Type, then populate $Table
    # with paths by type.  i.e. "Hardware&Disk Drive".  This is required
    # to insure proper lineage for devices.
    #
    if (Eq($Class->{Name}, $CL_HARDWARE) &&
	ArrangeIs({Name=>$AR_TYPE, Class=>$Class})) {
	my $What = sprintf("Generating type paths for %s . . .",
			   $Class->{Name});
	MeterCreate({Message=>$What});
	Status($What);

	my @List = sort keys %{ $Obj };
	for (my $i = 0; $i <= $#List; ++$i) {
	    if ($#List > 0) {
		MeterProgress({Percent=>$i/$#List});
	    }

	    my $type;
	    my $Path = PathGetByType({Name=>$List[$i],
				      Class=>$Class,
				      TypePtr=>\$type,
				      Obj=>$Obj});
	    if (!$Table->{$Path}) {
		Debug("TableAdd: Creating Table Entry for type <%s>=<%s>",
		      $Path, $type);
		$Table->{$Path}->{Obj} = $type;
	    }
	}
    }

    my $What = sprintf("Adding table data for %s . . .", $Class->{Name});
    MeterCreate({Message=>$What});
    Status($What);

    #
    # Build a hash table ($Table) with all the pathnames
    #
    my @List = sort keys %{ $Obj };
    for (my $i = 0; $i <= $#List; ++$i) {
	if ($#List > 0) {
	    MeterProgress({Percent=>$i/$#List});
	}

	my $Name = $List[$i];
	my $Entry = $Obj->{$Name};
	my $Path;
	if ($Entry =~ /mcGeneral/) {
	    $Path = $Class->{Name};
	} else {
	    $Path = PathGet({Entry=>$Entry, Obj=>$Obj, Name=>$Name,
			     Top=>$Class->{Name}});
	}
	Debug("TableAdd: Adding <%s>=<%s>", $Path, $Entry);
	$Table->{$Path}->{Obj} = $Entry;

	if ($Entry =~ /mcDevInfo/ && $Entry->Partitions) {
	    #
	    # Add partitions as child devices
	    #
	    my @List = $Entry->Partitions;
	    foreach my $Part (@List) {
		my $p = join($PathSep, $Path, $Part->Name);
		Debug("TableAdd: Adding <%s>=<%s>", $p, $Part);
		$Table->{$p}->{Obj} = $Part;
	    }
	} elsif ($Entry && $Entry =~ /mcDevInfo/ &&
		 $Entry->DevSpec =~ /mcCpuInfo/ &&
		 $Entry->DevSpec->Caches) {
	    #
	    # Add each cache as a child device
	    #
	    my @List = $Entry->DevSpec->Caches;
	    foreach my $Cache (@List) {
		my $p = join($PathSep, $Path, $Cache->Name);
		Debug("TableAdd: Adding <%s>=<%s>", $p, $Cache);
		$Table->{$p}->{Obj} = $Cache;
	    }
	}
    }

    # Save it
    uiClass::ClassTable({Class=>$Class, Set=>$Table});

    Status("");
}

1;
